import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;


/**
 * A Canvas class for doing basic graphical drawing.
 * This particular canvas focuses on drawing shapes that have been "added" to it.
 * 
 * @author Joel Ross
 * @author based on class by Michael Kolling and Bruce Quig
 *
 * @version Jan 2013
 */

public class ShapeCanvas
{
    private JFrame frame; //the window on which the drawing is displayed
    private ArrayList<DrawableShape> shapes; //the collection of shapes that are being drawn

    private Graphics2D g2d; //the graphics context we are drawing on
    private BufferedImage canvasImage; //an "image" that holds the current drawing
    private ShapePanel panel; //the component that is drawn inside the frame

    private Timer clock; //a timer for automatically updating the canvas as shapes are changed

    /**
     * Create a Canvas. Makes the Canvas visible when it is constructed
     * @param title  title to appear in Canvas Frame
     * @param width  the desired width for the canvas
     * @param height  the desired height for the canvas
     */
    public ShapeCanvas(String title, int width, int height)
    {
        frame = new JFrame(); //new JFrame
        frame.setTitle(title);
        //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); //exit program when closed

        //initialize the panel and drawing images
        panel = new ShapePanel(); 
        panel.setPreferredSize(new Dimension(width, height));
        canvasImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        g2d = canvasImage.createGraphics();
        g2d.setStroke(new BasicStroke(2));
        frame.getContentPane().add(panel); //add the panel to the frame

        shapes = new ArrayList<DrawableShape>(); //initialize the list

        frame.pack(); //pack everything
        frame.setVisible(true); //show on the screen

        clock = new Timer(20, new ActionListener() { //this line of code makes a new ActionListener that refreshes every tick
            public void actionPerformed(ActionEvent e) { refresh(); }
        });
        clock.start(); //start the clock;
    }

    /**
     * Create a Canvas with default height and width (400 x 400)
     * @param title  title to appear in Canvas Frame
     */
    public ShapeCanvas(String title)
    {
        this(title, 400, 400);        
    }

    /**
     * Create a Canvas with default height and width (400 x 400) and the title "ShapeCanvas".
     */
    public ShapeCanvas()
    {
        this("ShapeCanvas");
    }
    
    /**
     * Adds a Shape to the canvas. Any changes to those shapes will be reflected
     * on the drawn canvas.
     * @param shape the Shape to add to the canvas
     */
    public void addShape(DrawableShape shape)
    {
        shapes.add(shape);
    }

    /**
     * Removes a Shape from the canvas. It will no longer be shown.
     * @param shape the Shape to remove from the canvas
     */
    public void removeShape(DrawableShape shape)
    {
        shapes.remove(shape);
    }

    /**
     * A method that refreshes what is displayed on the canvas.
     * This method redraws the shapes on the canvas.
     */
    public void refresh()
    {
        g2d.setColor(Color.WHITE);
        g2d.fillRect(0,0,canvasImage.getWidth(), canvasImage.getHeight());
        for(DrawableShape shape : shapes)
        {
            shape.draw(g2d);
        }
        panel.repaint();
    }

    /**
     * Shows the canvas frame, in case it was closed.
     */
    public void makeVisible()
    {
        frame.setVisible(true);
    }
    
    /**
     * Toggles whether the canvas should automatically refresh.
     * On by default.
     */
    private void toggleAutomaticRefresh()
    {
        if(clock.isRunning())
        {
            clock.stop();
        }
        else
        {
            clock.start();
        }
    }

    /************************************************************************
     * Inner class ShapePanel - the actual canvas component contained in the
     * Canvas frame. This is essentially a JPanel that draws the canvasImage on it
     */
    private class ShapePanel extends JPanel
    {
        public void paintComponent(Graphics g)
        {
            g.drawImage(canvasImage,0,0,null); //draw the canvasImage on this component's graphics context
        }
    }
}


